@@ -71,7 +71,7 @@ class FileTemplatesHandler(APIHandler):
         filename = cm.increment_filename(filename, path)
         fullpath = url_path_join(path, filename)
 
-        realpath = Path(cm.root_dir) / url2path(fullpath)
+        realpath = Path(cm.root_dir).absolute() / url2path(fullpath)
         if not realpath.parent.exists():
             realpath.parent.mkdir(parents=True)
 
@@ -113,7 +113,7 @@ class ProjectsHandler(APIHandler):
         Returns:
             Path: Absolute path
         """
-        return Path(self.contents_manager.root_dir) / url2path(path)
+        return Path(self.contents_manager.root_dir).absolute() / url2path(path)
 
     @tornado.web.authenticated
     async def get(self, path: str = ""):
@@ -141,11 +141,7 @@ class ProjectsHandler(APIHandler):
             )
 
         configuration = None
-        if len(path) == 0:
-            # Close the current open project
-            pass
-
-        else:
+        if len(path) != 0:
             configuration = dict()
             fullpath = self._get_realpath(path)
             # Check that the path is a project
@@ -161,6 +157,17 @@ class ProjectsHandler(APIHandler):
             else:
                 configuration["path"] = path
 
+        if self.template.conda_pkgs is not None and self.template.filter_kernel:
+            if len(path) == 0:
+                # Close the current open project
+                self.log.debug(f"[jupyter-project] Clear Kernel whitelist")
+                self.kernel_spec_manager.whitelist = set()
+            elif "environment" in configuration:
+                kernelspecs = self.kernel_spec_manager.get_all_specs()
+                kernels = {n for n, s in kernelspecs.items() if s["spec"]["metadata"].get("conda_env_name") == configuration["environment"]}
+                self.log.debug(f"[jupyter-project] Set Kernel whitelist to {kernels}")
+                self.kernel_spec_manager.whitelist = kernels
+
         self.finish(json.dumps({"project": configuration}))
 
     @tornado.web.authenticated
@@ -257,17 +264,22 @@ class SettingsHandler(APIHandler):
 
         Return body:
         {
-            "fileTemplates": [
+            fileTemplates: [
                 {
-                    "endpoint": str,
-                    "destination": str | null,
-                    "schema": JSONschema | null
+                    destination: str | null,
+                    endpoint: str,
+                    icon: str | null,
+                    name: str,
+                    schema: JSONschema | null
                 }
             ],
-            "projectTemplate": {
+            projectTemplate: {
                 configurationFilename: str,
+                defaultCondaPackages: str | null,
                 defaultPath: str | null,
-                schema: JSONschema | null
+                editableInstall: bool,
+                schema: JSONschema | null,
+                withGit: bool
             }
         }
         """
@@ -387,6 +399,7 @@ def setup_handlers(
             "schema": (
                 project_template.schema if len(project_template.schema) else None
             ),
+            "withGit": True,  # TODO make it configurable
         }
 
     handlers.append(
