@@ -5,22 +5,24 @@ import {
 import { ICommandPalette, IThemeManager } from '@jupyterlab/apputils';
 import { IStateDB } from '@jupyterlab/coreutils';
 import { IFileBrowserFactory } from '@jupyterlab/filebrowser';
+import { IGitExtension } from '@jupyterlab/git';
 import { ILauncher } from '@jupyterlab/launcher';
 import { IMainMenu } from '@jupyterlab/mainmenu';
 import { IStatusBar } from '@jupyterlab/statusbar';
 import { defaultIconRegistry } from '@jupyterlab/ui-components';
+import { IEnvironmentManager } from 'jupyterlab_conda';
 import { activateFileGenerator } from './filetemplates';
 import { requestAPI } from './jupyter-project';
 import { activateProjectManager } from './project';
 import { registerIcons } from './style';
 import { setCurrentTheme } from './theme';
-import { IProjectManager, PluginID, Templates } from './tokens';
+import { IProjectManager, PLUGIN_ID, Templates } from './tokens';
 
 /**
  * Initialization data for the jupyter-project extension.
  */
 const extension: JupyterFrontEndPlugin<IProjectManager> = {
-  id: PluginID,
+  id: PLUGIN_ID,
   autoStart: true,
   activate: async (
     app: JupyterFrontEnd,
@@ -30,7 +32,9 @@ const extension: JupyterFrontEndPlugin<IProjectManager> = {
     launcher: ILauncher | null,
     menu: IMainMenu | null,
     statusbar: IStatusBar | null,
-    themeManager: IThemeManager | null
+    themeManager: IThemeManager | null,
+    condaManager: IEnvironmentManager | null,
+    git: IGitExtension | null
   ): Promise<IProjectManager> => {
     const { commands } = app;
 
@@ -51,6 +55,8 @@ const extension: JupyterFrontEndPlugin<IProjectManager> = {
           browserFactory,
           settings.projectTemplate,
           palette,
+          condaManager,
+          git,
           launcher,
           menu,
           statusbar
@@ -71,7 +77,7 @@ const extension: JupyterFrontEndPlugin<IProjectManager> = {
 
       console.log('JupyterLab extension jupyter-project is activated!');
     } catch (error) {
-      console.error(`Fail to activate ${PluginID}`, error);
+      console.error(`Fail to activate ${PLUGIN_ID}`, error);
     }
 
     app.restored.then(() => {
@@ -84,7 +90,14 @@ const extension: JupyterFrontEndPlugin<IProjectManager> = {
     return manager;
   },
   requires: [ICommandPalette, IFileBrowserFactory, IStateDB],
-  optional: [ILauncher, IMainMenu, IStatusBar, IThemeManager]
+  optional: [
+    ILauncher,
+    IMainMenu,
+    IStatusBar,
+    IThemeManager,
+    IEnvironmentManager,
+    IGitExtension
+  ]
 };
 
 export default extension;
