@@ -2,20 +2,26 @@ import { JupyterFrontEnd } from '@jupyterlab/application';
 import {
   Dialog,
   ICommandPalette,
+  InputDialog,
   showDialog,
   showErrorMessage
 } from '@jupyterlab/apputils';
 import { IStateDB, PathExt, URLExt } from '@jupyterlab/coreutils';
 import { FileDialog, IFileBrowserFactory } from '@jupyterlab/filebrowser';
+import { IGitExtension } from '@jupyterlab/git';
 import { ILauncher } from '@jupyterlab/launcher';
 import { IMainMenu } from '@jupyterlab/mainmenu';
 import { Contents } from '@jupyterlab/services';
 import { IStatusBar } from '@jupyterlab/statusbar';
 import { CommandRegistry } from '@phosphor/commands';
-import { JSONExt, ReadonlyJSONObject } from '@phosphor/coreutils';
+import {
+  JSONExt,
+  PromiseDelegate,
+  ReadonlyJSONObject
+} from '@phosphor/coreutils';
 import { Signal, Slot } from '@phosphor/signaling';
 import { Menu } from '@phosphor/widgets';
-import { IEnvironmentManager, Conda } from 'jupyterlab_conda';
+import { Conda, IEnvironmentManager } from 'jupyterlab_conda';
 import { INotification } from 'jupyterlab_toastify';
 import JSONSchemaBridge from 'uniforms-bridge-json-schema';
 import YAML from 'yaml';
@@ -29,6 +35,7 @@ import {
   Project,
   Templates
 } from './tokens';
+import { ForeignCommandIDs, renderStringTemplate } from './utils';
 import { createValidator } from './validator';
 
 /**
@@ -44,16 +51,6 @@ const FORBIDDEN_ENV_CHAR = /[/\s:#]/gi;
  */
 const STATE_ID = `${PLUGIN_ID}:project`;
 
-/**
- * Namespace of foreign command IDs used
- */
-namespace ForeignCommandIDs {
-  export const closeAll = 'application:close-all';
-  export const goTo = 'filebrowser:go-to-path';
-  export const openPath = 'filebrowser:open-path';
-  export const saveAll = 'docmanager:save-all';
-}
-
 /**
  * Namespace for Conda
  */
@@ -69,7 +66,7 @@ namespace CondaEnv {
     /**
      * Packages list
      */
-    dependencies: string[];
+    dependencies?: string[];
     /**
      * Environment name
      */
@@ -126,10 +123,13 @@ class ProjectManager implements IProjectManager {
         if (this.project) {
           this.open(this.project.path);
         }
+        this._restored.resolve();
       })
       .catch(error => {
-        console.error('Unable to restore saved project.', error);
+        const message = 'Unable to restore saved project.';
+        console.error(message, error);
         this.reset();
+        this._restored.reject(message);
       });
   }
 
@@ -151,7 +151,11 @@ class ProjectManager implements IProjectManager {
    * Default path to open in a project
    */
   get defaultPath(): string {
-    return this._defaultPath;
+    let defaultPath = this._defaultPath;
+    if (this.project) {
+      defaultPath = renderStringTemplate(defaultPath, this.project);
+    }
+    return defaultPath;
   }
 
   /**
@@ -176,6 +180,13 @@ class ProjectManager implements IProjectManager {
     return this._projectChanged;
   }
 
+  /**
+   * A promise resolved when the project state has been restored.
+   */
+  get restored(): Promise<void> {
+    return this._restored.promise;
+  }
+
   /**
    * Schema to be handled by the form
    */
@@ -183,13 +194,6 @@ class ProjectManager implements IProjectManager {
     return this._schema;
   }
 
-  /**
-   * Should we synchronize an conda environment with the project
-   */
-  get withConda(): boolean {
-    return this.defaultCondaPackages ? true : false;
-  }
-
   /**
    * Generate a new project in path
    *
@@ -309,10 +313,23 @@ class ProjectManager implements IProjectManager {
   private _editableInstall = true;
   private _project: Project.IModel | null = null;
   private _projectChanged = new Signal<this, Project.IChangedArgs>(this);
+  private _restored = new PromiseDelegate<void>();
   private _schema: JSONSchemaBridge | null = null;
   private _state: IStateDB;
 }
 
+/**
+ * Build the project info from project model
+ * @param project The project model
+ * @returns The project info
+ */
+export function getProjectInfo(project: Project.IModel): Project.IInfo {
+  return {
+    ...project,
+    dirname: PathExt.dirname(project.path)
+  };
+}
+
 /**
  * Reset the current application workspace:
  * - Save all opened files
@@ -338,6 +355,8 @@ async function resetWorkspace(commands: CommandRegistry): Promise<void> {
  * @param browserFactory The file browser factory
  * @param settings The project template settings
  * @param palette The command palette
+ * @param condaManager The Conda extension service
+ * @param git The Git extension service
  * @param launcher The application launcher
  * @param menu The application menu
  * @param statusbar The application status bar
@@ -349,13 +368,22 @@ export function activateProjectManager(
   settings: Templates.IProject,
   palette: ICommandPalette,
   condaManager: IEnvironmentManager | null,
+  git: IGitExtension | null,
   launcher: ILauncher | null,
   menu: IMainMenu | null,
   statusbar: IStatusBar | null
 ): IProjectManager {
   const { commands, serviceManager } = app;
+  const filebrowser = browserFactory.defaultBrowser.model;
   const category = 'Project';
 
+  if (!settings.defaultCondaPackages) {
+    condaManager = null;
+  }
+  if (!settings.withGit) {
+    git = null;
+  }
+
   // Cannot blocking wait for the application otherwise this will bock
   // the all application at launch time
   const manager = new ProjectManager(settings, state, app.restored);
@@ -402,11 +430,84 @@ export function activateProjectManager(
     });
   }
 
+  // Update the conda environment when git HEAD changes
+  const gitSlot: Slot<IGitExtension, void> = git => {
+    if (
+      condaManager &&
+      manager.project &&
+      manager.project.environment &&
+      // TODO git path handling is not consistent with jupyter ecosystem...
+      '/' + git.getRelativeFilePath() === manager.project.path
+    ) {
+      const envName = manager.project.environment;
+      const branch = git.currentBranch ? git.currentBranch.name : 'unknown';
+      const errorMsg = `Fail to update conda environment after git HEAD changed on branch ${branch}`;
+      let toastId: React.ReactText = null;
+      Private.compareSpecification(
+        condaManager,
+        manager.project,
+        serviceManager.contents
+      )
+        .then(async ({ isIdentical, file, notInFile }) => {
+          if (!isIdentical && file) {
+            toastId = await INotification.inProgress(
+              `Updating environment for branch ${branch}...`
+            );
+            condaManager
+              .getPackageManager()
+              .packageChanged.disconnect(condaSlot);
+            try {
+              toastId = await Private.updateEnvironment(
+                envName,
+                file,
+                notInFile,
+                condaManager,
+                toastId
+              );
+            } finally {
+              condaManager
+                .getPackageManager()
+                .packageChanged.connect(condaSlot);
+            }
+            if (toastId) {
+              return INotification.update({
+                toastId,
+                message: `Environment ${envName} updated for branch ${branch}`,
+                type: 'success',
+                autoClose: 5000
+              });
+            }
+          }
+        })
+        .catch(error => {
+          console.error(errorMsg, error);
+          if (toastId) {
+            INotification.update({
+              toastId,
+              message: errorMsg,
+              type: 'error'
+            });
+          }
+        });
+    }
+  };
+
+  manager.restored.then(() => {
+    if (manager.project && condaManager) {
+      // Apply kernel whitelist
+      serviceManager.sessions.refreshSpecs();
+
+      condaManager.getPackageManager().packageChanged.connect(condaSlot);
+      if (git) {
+        git.headChanged.connect(gitSlot);
+      }
+    }
+  });
+
   commands.addCommand(CommandIDs.newProject, {
     caption: 'Create a new project.',
     execute: async args => {
-      const cwd: string =
-        (args['cwd'] as string) || browserFactory.defaultBrowser.model.path;
+      const cwd: string = (args['cwd'] as string) || filebrowser.path;
       let toastId = args['toastId'] as React.ReactText;
       const cleanToast = toastId === undefined;
 
@@ -423,7 +524,7 @@ export function activateProjectManager(
       }
 
       try {
-        const message = 'Creating project.';
+        const message = 'Creating project...';
         if (toastId) {
           INotification.update({
             toastId,
@@ -434,11 +535,40 @@ export function activateProjectManager(
         }
         const model = await manager.create(cwd, params);
 
+        // Initialize as Git repository
+        if (git) {
+          try {
+            INotification.update({
+              toastId,
+              message: 'Initializing Git...'
+            });
+            // TODO @jupyterlab/git does not respect frontend path...
+            // await git.init(model.path);
+            await filebrowser.cd(model.path);
+            await commands.execute(ForeignCommandIDs.gitInit);
+          } catch (error) {
+            console.error(
+              'Fail to initialize the project as Git repository.',
+              error
+            );
+          }
+        }
+
         await commands.execute(CommandIDs.openProject, {
           path: model.path,
           toastId
         });
 
+        if (git) {
+          try {
+            // Add all files and commit
+            await git.addAllUntracked();
+            await git.commit(`Initialize project ${model.name}`);
+          } catch (error) {
+            console.error('Fail to commit the project files.', error);
+          }
+        }
+
         if (cleanToast) {
           INotification.update({
             toastId,
@@ -466,6 +596,80 @@ export function activateProjectManager(
     label: args => (!args['isLauncher'] ? 'New Project' : 'New')
   });
 
+  commands.addCommand(CommandIDs.importProject, {
+    caption: 'Import a project by cloning a Git repository',
+    execute: async args => {
+      if (!git) {
+        showErrorMessage(
+          'Git extension not available',
+          'The `@jupyterlab/git` extension is not installed.'
+        );
+        return;
+      }
+
+      const path: string = (args['cwd'] as string) || filebrowser.path;
+      let toastId = args['toastId'] as React.ReactText;
+      const cleanToast = toastId === undefined;
+
+      let url = args['url'] as string;
+      if (!url) {
+        const answer = await InputDialog.getText({
+          title: 'URL of the Git repository to import',
+          placeholder: 'https://my.git.server/project/repository.git'
+        });
+
+        if (!answer.button.accept) {
+          return;
+        }
+
+        url = answer.value;
+      }
+
+      try {
+        const message = 'Importing project...';
+        if (toastId) {
+          INotification.update({
+            toastId,
+            message
+          });
+        } else {
+          toastId = await INotification.inProgress(message);
+        }
+
+        await git.clone(path, url);
+
+        const folderName = PathExt.basename(url, 'git');
+        await commands.execute(CommandIDs.openProject, {
+          path: PathExt.join(path, folderName),
+          toastId
+        });
+
+        if (cleanToast) {
+          INotification.update({
+            toastId,
+            message: `Project successfully import from '${url}'.`,
+            type: 'success',
+            autoClose: 5000
+          });
+        }
+      } catch (error) {
+        const message = 'Fail to import the project';
+        await manager.close();
+        console.error(message, error);
+
+        INotification.update({
+          toastId,
+          message,
+          type: 'error'
+        });
+      }
+    },
+    iconClass: args =>
+      args['isLauncher'] ? 'jp-JupyterProjectProjectIcon' : '',
+    isVisible: () => git !== null,
+    label: args => (!args['isLauncher'] ? 'Import Project' : 'Import')
+  });
+
   commands.addCommand(CommandIDs.openProject, {
     label: args => (!args['isLauncher'] ? 'Open Project' : 'Open'),
     caption: 'Open a project',
@@ -493,8 +697,8 @@ export function activateProjectManager(
         // From the user through an open file dialog
         const result = await FileDialog.getOpenFiles({
           filter: value => value.name === manager.configurationFilename,
-          iconRegistry: browserFactory.defaultBrowser.model.iconRegistry,
-          manager: browserFactory.defaultBrowser.model.manager,
+          iconRegistry: filebrowser.iconRegistry,
+          manager: filebrowser.manager,
           title: 'Select the project file'
         });
 
@@ -529,8 +733,11 @@ export function activateProjectManager(
           });
         }
 
-        if (condaManager && manager.withConda) {
+        if (condaManager) {
           condaManager.getPackageManager().packageChanged.disconnect(condaSlot);
+          if (git) {
+            git.headChanged.disconnect(gitSlot);
+          }
           toastId = await Private.openProject(
             manager,
             serviceManager.contents,
@@ -538,7 +745,14 @@ export function activateProjectManager(
             commands,
             toastId
           );
+          // Re-open to set the kernel whitelist
+          if (manager.project.environment) {
+            await manager.open(manager.project.path);
+          }
           condaManager.getPackageManager().packageChanged.connect(condaSlot);
+          if (git) {
+            git.headChanged.connect(gitSlot);
+          }
 
           // Force refreshing session to take into account the new environment
           serviceManager.sessions.refreshSpecs();
@@ -578,10 +792,16 @@ export function activateProjectManager(
     execute: async () => {
       try {
         await resetWorkspace(commands);
-        manager.close();
-        // TODO Clean kernel white list
+        await manager.close();
         if (condaManager) {
+          // Force refreshing session to take into account the whitelist suppression
+          serviceManager.sessions.refreshSpecs();
+
           condaManager.getPackageManager().packageChanged.disconnect(condaSlot);
+
+          if (git) {
+            git.headChanged.disconnect(gitSlot);
+          }
         }
       } catch (error) {
         showErrorMessage('Failed to close the current project', error);
@@ -621,6 +841,9 @@ export function activateProjectManager(
         toastId = null;
       }
       if (condaEnvironment && condaManager) {
+        // Force refreshing session to take into account the whitelist suppression
+        serviceManager.sessions.refreshSpecs();
+
         // 2. Remove associated conda environment
         const message = `Removing conda environment '${condaEnvironment}'...`;
         if (toastId) {
@@ -630,11 +853,14 @@ export function activateProjectManager(
         }
 
         condaManager.getPackageManager().packageChanged.disconnect(condaSlot);
+        if (git) {
+          git.headChanged.disconnect(gitSlot);
+        }
         try {
           await condaManager.remove(condaEnvironment);
 
-          // Force refreshing session to take into account the new environment
-          await serviceManager.sessions.refreshSpecs();
+          // Force refreshing session to take into account the removed environment
+          serviceManager.sessions.refreshSpecs();
         } catch (error) {
           const message = `Failed to remove the project environment ${condaEnvironment}`;
           console.error(message, error);
@@ -656,7 +882,11 @@ export function activateProjectManager(
 
   if (launcher) {
     // Add Project Cards
-    [CommandIDs.newProject, CommandIDs.openProject].forEach(command => {
+    [
+      CommandIDs.newProject,
+      CommandIDs.openProject,
+      CommandIDs.importProject
+    ].forEach(command => {
       launcher.add({
         command,
         args: { isLauncher: true },
@@ -667,6 +897,7 @@ export function activateProjectManager(
 
   const projectCommands = [
     CommandIDs.newProject,
+    CommandIDs.importProject,
     CommandIDs.openProject,
     CommandIDs.closeProject,
     CommandIDs.deleteProject
@@ -753,26 +984,13 @@ namespace Private {
       environmentName = foundEnvironment.name;
 
       if (!isIdentical && file) {
-        INotification.update({
-          toastId,
-          message: `Updating conda environment ${environmentName}... Please wait`
-        });
-
-        try {
-          // 1. Remove package not in the environment specification file
-          if (notInFile && notInFile.length > 0) {
-            await conda
-              .getPackageManager()
-              .remove(notInFile, foundEnvironment.name);
-          }
-          // 2. Update the environment according to the file
-          await conda.update(foundEnvironment.name, file, ENVIRONMENT_FILE);
-        } catch (error) {
-          const message = `Fail to update environment ${foundEnvironment.name}`;
-          console.error(message, error);
-          INotification.update({ toastId, message });
-          toastId = null;
-        }
+        toastId = await updateEnvironment(
+          environmentName,
+          file,
+          notInFile,
+          conda,
+          toastId
+        );
       }
     } else {
       // Import an environment
@@ -802,7 +1020,7 @@ namespace Private {
       } catch (error) {
         const message = `Fail to create the environment for ${model.name}`;
         console.error(message, error);
-        INotification.update({ toastId, message });
+        INotification.update({ toastId, message, type: 'error' });
         return null;
       }
     }
@@ -823,15 +1041,46 @@ namespace Private {
 
     // Update the config file
     const filePath = PathExt.join(model.path, manager.configurationFilename);
+    // Remove `path` if it exists - as it is user specific
+    const toSave = { ...model };
+    delete toSave.path;
     await contentService.save(filePath, {
       type: 'file',
       format: 'text',
-      content: JSON.stringify(model)
+      content: JSON.stringify(toSave)
     });
 
     return toastId;
   }
 
+  export async function updateEnvironment(
+    environmentName: string,
+    file: string,
+    notInFile: Set<string>,
+    conda: IEnvironmentManager,
+    toastId: React.ReactText
+  ): Promise<React.ReactText | null> {
+    INotification.update({
+      toastId,
+      message: `Updating conda environment ${environmentName}... Please wait`
+    });
+
+    try {
+      // 1. Remove package not in the environment specification file
+      if (notInFile && notInFile.size > 0) {
+        await conda.getPackageManager().remove([...notInFile], environmentName);
+      }
+      // 2. Update the environment according to the file
+      await conda.update(environmentName, file, ENVIRONMENT_FILE);
+    } catch (error) {
+      const message = `Fail to update environment ${environmentName}`;
+      console.error(message, error);
+      INotification.update({ toastId, message, type: 'error' });
+      toastId = null;
+    }
+    return toastId;
+  }
+
   export async function updateEnvironmentSpec(
     project: Project.IModel | null,
     condaManager: IEnvironmentManager | null,
@@ -892,27 +1141,42 @@ namespace Private {
     isIdentical: boolean;
     conda?: string;
     file?: string;
-    notInFile?: string[];
+    notInFile?: Set<string>;
   }> {
     let conda: string;
-    let condaPkgs: string[];
+    let condaPkgs: Set<string>;
     if (condaManager && project && project.environment) {
-      const description = await condaManager.export(project.environment, true);
-      const specification = YAML.parse(
-        await description.text()
-      ) as CondaEnv.IEnvSpecs;
-      condaPkgs = specification.dependencies
-        .sort()
-        .map(name => PACKAGE_NAME.exec(name)[0]);
-      // Clean the specification from environment name and prefix
-      delete specification.name;
-      delete specification.prefix;
-      conda = YAML.stringify(specification);
+      try {
+        // Does not raise any error if the environment does not exist, but dependencies will be absent.
+        const description = await condaManager.export(
+          project.environment,
+          true
+        );
+        const specification = YAML.parse(
+          await description.text()
+        ) as CondaEnv.IEnvSpecs;
+        if (specification.dependencies) {
+          condaPkgs = new Set(
+            specification.dependencies
+              .sort()
+              .map(name => PACKAGE_NAME.exec(name)[0])
+          );
+        }
+        // Clean the specification from environment name and prefix
+        delete specification.name;
+        delete specification.prefix;
+        conda = YAML.stringify(specification);
+      } catch (error) {
+        console.debug(
+          `Fail to list the packages for conda environment ${project.environment}`,
+          error
+        );
+      }
     }
 
     const specPath = PathExt.join(project.path, ENVIRONMENT_FILE);
     let file: string;
-    let filePkgs: string[];
+    let filePkgs: Set<string>;
     try {
       const m = await contents.get(specPath, {
         content: true,
@@ -920,9 +1184,13 @@ namespace Private {
         type: 'file'
       });
       const specification = YAML.parse(m.content) as CondaEnv.IEnvSpecs;
-      filePkgs = specification.dependencies
-        .sort()
-        .map(name => PACKAGE_NAME.exec(name)[0]);
+      if (specification.dependencies) {
+        filePkgs = new Set(
+          specification.dependencies
+            .sort()
+            .map(name => PACKAGE_NAME.exec(name)[0])
+        );
+      }
       if (specification.name) {
         delete specification.name;
       }
@@ -935,31 +1203,7 @@ namespace Private {
     }
 
     const isIdentical = conda === file;
-    let notInFile: string[];
-    if (!isIdentical && condaPkgs) {
-      if (filePkgs) {
-        notInFile = new Array<string>();
-        let fileI = 0;
-        let filePkg = filePkgs[fileI];
-        for (const condaPkg of condaPkgs) {
-          if (condaPkg < filePkg) {
-            notInFile.push(condaPkg);
-          } else {
-            while (condaPkg >= filePkg) {
-              if (fileI < filePkgs.length - 1) {
-                fileI += 1;
-                filePkg = filePkgs[fileI];
-              } else {
-                notInFile.push(condaPkg);
-                break;
-              }
-            }
-          }
-        }
-      } else {
-        notInFile = condaPkgs;
-      }
-    }
+    const notInFile = new Set([...condaPkgs].filter(pkg => !filePkgs.has(pkg)));
 
     return { isIdentical, conda, file, notInFile };
   }
